package com.lantone.qc.kernel.catalogue.behospitalized;

import com.lantone.qc.kernel.catalogue.QCCatalogue;
import com.lantone.qc.pub.model.InputInfo;
import com.lantone.qc.pub.model.OutputInfo;
import com.lantone.qc.pub.model.entity.Diag;
import com.lantone.qc.pub.model.entity.Family;
import com.lantone.qc.pub.model.label.FamilyLabel;
import com.lantone.qc.pub.util.StringUtil;
import org.springframework.stereotype.Component;

import java.util.List;

/**
 * @ClassName : BEH0074
 * @Description : 两系三代内是否有肿瘤病史未描述
 * @Author : 胡敬
 * @Date: 2020-03-11 17:05
 */
@Component
public class BEH0074 extends QCCatalogue {
    @Override
    protected void start(InputInfo inputInfo, OutputInfo outputInfo) {
        status.set("0");
        if (inputInfo.getBeHospitalizedDoc() != null && inputInfo.getBeHospitalizedDoc().getFamilyLabel() != null) {
            FamilyLabel familyLabel = inputInfo.getBeHospitalizedDoc().getFamilyLabel();
            List<Family> families = familyLabel.getFamilies();
            boolean findCancer = false;
            for (Family family : families) {
                if (family.getDiags() == null || family.getDiags().size() == 0) {
                    continue;
                }
                for (Diag diag : family.getDiags()) {
                    if (StringUtil.isNotBlank(diag.getName()) && (diag.getName().contains("癌") || diag.getName().contains("肿瘤"))
                            ||
                            StringUtil.isNotBlank(diag.getHospitalDiagName()) && (diag.getHospitalDiagName().contains("癌") || diag.getHospitalDiagName().contains("肿瘤"))) {
                        findCancer = true;
                        break;
                    }
                }
                if (findCancer) {
                    break;
                }
            }
            if (!findCancer) {
                status.set("-1");
            }
        }
    }
}