package com.lantone.qc.kernel.catalogue.behospitalized;

import com.lantone.qc.kernel.catalogue.QCCatalogue;
import com.lantone.qc.pub.model.InputInfo;
import com.lantone.qc.pub.model.OutputInfo;
import com.lantone.qc.pub.model.doc.BeHospitalizedDoc;
import com.lantone.qc.pub.model.doc.FirstCourseRecordDoc;
import com.lantone.qc.pub.model.entity.Clinical;
import com.lantone.qc.pub.model.entity.Diag;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Component;

import java.util.List;


/**
 * @ClassName : BEH0449
 * @Description : 现病史主症状与主诉症状不一致
 * @Author : 楼辉荣
 * @Date: 2020-03-06 17:28
 */
@Component
public class BEH0449 extends QCCatalogue {
    public void start(InputInfo inputInfo, OutputInfo outputInfo) {
        status.set("0");
        BeHospitalizedDoc beHospitalizedDoc = inputInfo.getBeHospitalizedDoc();
        if(beHospitalizedDoc != null){
            String chief_text = beHospitalizedDoc.getChiefLabel().getText();
            if(chief_text.contains("检查") || chief_text.contains("术后") || chief_text.contains("药物")){
                status.set("0");
            }else {
                List<Clinical> clinicals_chief = beHospitalizedDoc.getChiefLabel().getClinicals();
                List<Clinical> clinicals_present = beHospitalizedDoc.getPresentLabel().getClinicals();
                if(clinicals_chief != null && clinicals_present !=null){
                    Clinical clinical_chief = clinicals_chief.get(0);
                    Clinical clinical_present = clinicals_present.get(0);
                    if(!clinical_chief.getName().equals(clinical_present.getName())){
                        status.set("-1");
                    }
                }
            }

            String chiefDiag = "";
            List<Diag> diags = beHospitalizedDoc.getChiefLabel().getDiags();
            if (diags != null && diags.size() > 0) {
                chiefDiag = diags.get(0).getHospitalDiagName();
            }
            String presentDiag = "";
            diags = beHospitalizedDoc.getPresentLabel().getDiags();
            if (diags != null && diags.size() > 0) {
                presentDiag = diags.get(0).getHospitalDiagName();
            }
            if (StringUtils.isNotEmpty(chiefDiag) && StringUtils.isNotEmpty(presentDiag) && chiefDiag.equals(presentDiag)) {
                status.set("0");
            }
        }
    }
}