package com.lantone.qc.kernel.catalogue.threelevelward;

import com.lantone.qc.kernel.catalogue.QCCatalogue;
import com.lantone.qc.kernel.util.CatalogueUtil;
import com.lantone.qc.pub.Content;
import com.lantone.qc.pub.model.InputInfo;
import com.lantone.qc.pub.model.OutputInfo;
import com.lantone.qc.pub.model.doc.ThreeLevelWardDoc;
import com.lantone.qc.pub.util.DateUtil;
import com.lantone.qc.pub.util.StringUtil;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * @ClassName : THR0128
 * @Description : 每周无3次主治医师查房记录
 * @Author : 胡敬
 * @Date: 2020-03-19 16:52
 */
@Component
public class THR0128 extends QCCatalogue {
    public void start(InputInfo inputInfo, OutputInfo outputInfo) {
        status.set("0");
        if (inputInfo.getBeHospitalizedDoc() != null && inputInfo.getBeHospitalizedDoc().getStructureMap() != null
                && inputInfo.getThreeLevelWardDocs().size() > 0) {
            Map<String, String> beHospitalStructureMap = inputInfo.getBeHospitalizedDoc().getStructureMap();
            String admisTime = beHospitalStructureMap.get(Content.admisDate);
            if (CatalogueUtil.isEmpty(admisTime)) {
                return;
            }
            List<ThreeLevelWardDoc> threeLevelWardDocs = inputInfo.getThreeLevelWardDocs();
            int hoursPerWeek = 7 * 24 * 60;
            String roundRecordThisWeek = "";
            List<String> roundRecordEveryWeek = new ArrayList<>();
            //开始时间(入院时间)
            Date beginDate = StringUtil.parseDateTime(admisTime);
            int i = 1;
            //每周的病历记录
            while (i >= 1) {
                roundRecordThisWeek = extractWardRecord(threeLevelWardDocs, beginDate, hoursPerWeek);
                if (CatalogueUtil.isEmpty(roundRecordThisWeek)) {
                    break;
                }
                roundRecordEveryWeek.add(roundRecordThisWeek);
                beginDate = DateUtil.addDate(beginDate, 7);
                i++;
            }
            if (roundRecordEveryWeek.size() == 0){
                status.set("0");
                return;
            }
            for (String roundRecord : roundRecordEveryWeek) {
                int indicationsNum = CatalogueUtil.appearNumber(roundRecord.split(","), Content.indications);
                if (indicationsNum < 3) {
                    //每周无3次主治医师查房记录
                    status.set("-1");
                    return;
                }
            }
        }
    }

    /**
     * 抽取duration分钟内所有查房标题
     * 抽取一周内所有查房标题，若一周内记录少于6天，则返回""
     *
     * @param threeLevelWardDocs
     * @param admisDate
     * @param duration
     * @return
     */
    private static String extractWardRecord(List<ThreeLevelWardDoc> threeLevelWardDocs, Date admisDate, int duration) {
        String recordTime = "", recordTitle = "";
        List<Date> dateList = new ArrayList();
        for (ThreeLevelWardDoc threeLevelWardDoc : threeLevelWardDocs) {
            Map<String, String> threeLevelWardStructureMap = threeLevelWardDoc.getStructureMap();
            recordTime = threeLevelWardStructureMap.get("查房日期");
            Date recordDate = StringUtil.parseDateTime(recordTime);
            if (recordDate == null) {
                continue;
            }
            if (admisDate.before(recordDate) && !CatalogueUtil.compareTime(admisDate, recordDate, Long.valueOf(duration))) {
                String wardTitle = CatalogueUtil.removeSpecialChar(threeLevelWardStructureMap.get("查房标题"));
                recordTitle += wardTitle + ",";
                dateList.add(recordDate);
            }
        }
        if (dateList.size() > 0) {
            dateList.sort(Date::compareTo);
            if (CatalogueUtil.compareTime(admisDate, dateList.get(dateList.size() - 1), Long.valueOf(6 * 24 * 60))) {
                return recordTitle;
            }
        }
        return "";
    }
}