package com.lantone.qc.kernel.catalogue.threelevelward;

import com.lantone.qc.kernel.catalogue.QCCatalogue;
import com.lantone.qc.pub.Content;
import com.lantone.qc.pub.model.InputInfo;
import com.lantone.qc.pub.model.OutputInfo;
import com.lantone.qc.pub.model.doc.ThreeLevelWardDoc;
import com.lantone.qc.pub.model.label.ThreeLevelWardLabel;
import com.lantone.qc.pub.util.StringUtil;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Component;

import java.util.List;

/**
 * @ClassName : THR0137
 * @Description : 上级医师（副主任医师/主任医师）首次查房无病情分析/评估
 * @Author : 胡敬
 * @Date: 2020-03-23 14:16
 */
@Component
public class THR0137 extends QCCatalogue {
    public void start(InputInfo inputInfo, OutputInfo outputInfo) {
        if (inputInfo.getThreeLevelWardDocs().size() > 0) {
            List<ThreeLevelWardDoc> threeLevelWardDocs = inputInfo.getThreeLevelWardDocs();
            for (ThreeLevelWardDoc threeLevelWardDoc : threeLevelWardDocs) {
                List<ThreeLevelWardLabel> threeLevelWardLabels = threeLevelWardDoc.getThreeLevelWardLabel();
                for (ThreeLevelWardLabel threeLevelWardLabel : threeLevelWardLabels) {
                    if (StringUtil.isBlank(threeLevelWardLabel.getTitle())
                            || !Content.director.equals(threeLevelWardLabel.getTitle())) {
                        continue;
                    }
                    if (threeLevelWardLabel.getDiags().size() > 0
                            && StringUtil.isNotBlank(threeLevelWardLabel.getDiagBasisText())
                            && (threeLevelWardLabel.getDiffDiag().size() > 0 || StringUtils.isNotEmpty(threeLevelWardLabel.getDiffDiagText()))
                            && threeLevelWardLabel.getTreatmentPlans().size() > 0) {
                        status.set("0");
                        return;
                    }
                }
            }
        }
    }
}