package com.lantone.qc.kernel.catalogue.threelevelward;

import com.lantone.qc.kernel.catalogue.QCCatalogue;
import com.lantone.qc.pub.model.InputInfo;
import com.lantone.qc.pub.model.OutputInfo;
import com.lantone.qc.pub.model.doc.ThreeLevelWardDoc;
import com.lantone.qc.pub.model.label.ThreeLevelWardLabel;
import com.lantone.qc.pub.util.ListUtil;
import com.lantone.qc.pub.util.StringUtil;
import org.springframework.stereotype.Component;

import java.util.List;

/**
 * @ClassName : THR0428
 * @Description : 危重患者上级医师(副高及以上)查房记录无病情分析及评估
 * @Author : Mark
 * @Date: 2020-04-05 16:03
 */
@Component
public class THR0428 extends QCCatalogue {
    public void start(InputInfo inputInfo, OutputInfo outputInfo) {
        status.set("0");
        if (ListUtil.isEmpty(inputInfo.getSeriouslyIllNoticeDocs())) {
            return;
        }
        if (inputInfo.getThreeLevelWardDocs().size() > 0) {
            List<ThreeLevelWardDoc> threeLevelWardDocs = inputInfo.getThreeLevelWardDocs();
            for (ThreeLevelWardDoc threeLevelWardDoc : threeLevelWardDocs) {
                List<ThreeLevelWardLabel> threeLevelWardLabels = threeLevelWardDoc.getThreeLevelWardLabel();
                for (ThreeLevelWardLabel threeLevelWardLabel : threeLevelWardLabels) {
                    if (threeLevelWardLabel.getSerious() != 1) {
                        continue;
                    }
                    if (threeLevelWardLabel.getDiags().size() == 0
                            || StringUtil.isNotBlank(threeLevelWardLabel.getDiagBasisText()) && threeLevelWardLabel.getDiagBasisText().length() < 30
                            || threeLevelWardLabel.getDiffDiag().size() == 0 && StringUtil.isBlank(threeLevelWardLabel.getDiffDiagText())
                            || threeLevelWardLabel.getTreatmentPlans().size() == 0) {
                        status.set("-1");
                        return;
                    }
                }
            }
        }
    }
}